type op = Add | Sub | Mult | Div| Equal | Neq | Less | Leq | Greater | Geq | Cmp | Concat | And | Or



type lexpr = 					(* Expressions *)
     Literal of string 				(* 42.0 *)
   | StrConst of string				(* "Hello World" *)
   | Param of string * string 			(* a->b *)
   | Uminus of lexpr				(* -42.0 *)
   | Binop of lexpr * op * lexpr 		(* a->d + b->c *)
   | AssignL of string * string * lexpr		(* a->b = 42 *)
	
type expr = 					(* Expressions *)
     Id of string 				(* a *)
   | AssignVar of string * expr			(* a = b or a=foo(b) *)
   | Assign of string * string * lexpr		(* a->b = 42 *)
   | Call of string * expr list 		(* foo(a, b) *)


type stmt = 					(* Statements *)
     Block of stmt list 			(* { ... } *)
   | Expr of expr 				(* a = b; *)
   | Return of expr 				(* return a; *)
   | If of lexpr * stmt * stmt 			(* if (a->b == 1) {} else {} *)
   | While of lexpr * stmt 			(* while (a->b<10) { a->b = a->b + 1 } *)
 

type param_decl = {
     p_key   	: string;
     p_value 	: string; 				
}

type v_decl = {
     v_name 	: string; 				
     v_params 	: param_decl list; 				
}  

type func_decl = {
     fname 	: string; 			(* Name of the function *)
     formals 	: string list; 			(* Formal argument names *)
     locals 	: v_decl list; 			(* Locally defined variables *)
     body 	: stmt list;			(* Body of the function *)
}




type program = v_decl list * func_decl list 	(* global vars, funcs *)