%{ open Ast %}

%token ARROW PLUS MINUS TIMES DIVIDE COMMA LPAREN RPAREN LBRACE RBRACE SEMI 
%token EQ NEQ LT LEQ GT GEQ COMPARE CONCAT AND OR RETURN WHILE IF ELSE INIT EOF
%token <string> LITERAL
%token <string> ID
%token <string> STRCONST

%token ASSIGN


%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%left AND OR
%left EQ NEQ
%left LT GT LEQ GEQ COMPARE
%left CONCAT
%left PLUS MINUS
%left TIMES DIVIDE
%nonassoc UMINUS



%start program
%type <Ast.program> program

%%

program:

     /* nothing */ 				{ [], [] }
   | program vdecl 				{ ($2 :: fst $1), snd $1 }
   | program fdecl 				{ fst $1, ($2 :: snd $1) }


fdecl:
    ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
    		   				{ { fname = $1;
						    formals = $3;
						    locals = List.rev $6;
						    body = List.rev $7 } }

formals_opt:
      /* nothing */ 				{ [] }
   | formal_list 				{ List.rev $1 }


formal_list:
     ID 					{ [$1] }
   | formal_list COMMA ID 			{ $3 :: $1 }


params_opt:
      /* nothing */ 				{ [] }
   | params_list 				{ $1 }


params_list:
     param 					{ [$1] }
   | params_list COMMA param 			{ $3 :: $1 }

param:
     LPAREN ID COMMA STRCONST RPAREN 		{ { 	p_key   = $2;
				  			p_value = String.sub $4 1 ((String.length $4)-2) ;} }
	
   | LPAREN ID COMMA MINUS LITERAL RPAREN 	{ { 	p_key   = $2;
				  			p_value = string_of_float (-.(float_of_string $5));} }

   | LPAREN ID COMMA LITERAL RPAREN 		{ { 	p_key   = $2;
				  			p_value = $4;} }
   

vdecl_list:
      /* nothing */ 				{ [] }
   | vdecl_list vdecl 				{ $2 :: $1 }


vdecl:
     INIT ID LPAREN params_opt RPAREN SEMI 	{ { 	v_name = $2;
				       			v_params = List.rev $4;} } 

 
stmt_list:
     /* nothing */ 				{ [] }
   | stmt_list stmt 				{ $2 :: $1 }


stmt:
	
     expr SEMI 					{ Expr($1) }
   | RETURN expr SEMI 				{ Return($2) }
   | LBRACE stmt_list RBRACE 			{ Block(List.rev $2) }
   | IF LPAREN lexpr RPAREN stmt %prec NOELSE 	{ If($3, $5, Block([])) }
   | IF LPAREN lexpr RPAREN stmt ELSE stmt 	{ If($3, $5, $7) }
   | WHILE LPAREN lexpr RPAREN stmt 		{ While($3, $5) }


expr:

     ID 					{ Id($1) }
   | ID ASSIGN expr				{ AssignVar($1, $3)} 
   | ID ARROW ID ASSIGN lexpr 			{ Assign($1, $3, $5) }   
   | ID LPAREN actuals_opt RPAREN 		{ Call($1, $3) }
   | LPAREN expr RPAREN 			{ $2 }
   

lexpr:

     LITERAL 					{ Literal($1) }
   | STRCONST					{ StrConst($1) } 
   | ID ARROW ID ASSIGN lexpr 			{ AssignL($1, $3, $5) } 
   | ID ARROW ID 				{ Param($1, $3) }
   | MINUS lexpr %prec UMINUS 			{ Uminus($2) }
   | lexpr PLUS lexpr 				{ Binop($1, Add, $3) }
   | lexpr MINUS lexpr				{ Binop($1, Sub, $3) }
   | lexpr TIMES lexpr 				{ Binop($1, Mult, $3) }
   | lexpr DIVIDE lexpr 			{ Binop($1, Div, $3) }
   | lexpr EQ lexpr 				{ Binop($1, Equal, $3) }
   | lexpr NEQ lexpr 				{ Binop($1, Neq, $3) }
   | lexpr LT lexpr 				{ Binop($1, Less, $3) }
   | lexpr LEQ lexpr 				{ Binop($1, Leq, $3) }
   | lexpr GT lexpr 				{ Binop($1, Greater, $3) }
   | lexpr GEQ lexpr 				{ Binop($1, Geq, $3) }
   | lexpr CONCAT lexpr 			{ Binop($1, Concat, $3) }
   | lexpr COMPARE lexpr 			{ Binop($1, Cmp, $3) }
   | lexpr AND lexpr 				{ Binop($1, And, $3) }
   | lexpr OR lexpr 				{ Binop($1, Or, $3) }
   | LPAREN lexpr RPAREN 			{ $2 }
	

actuals_opt:
     /* nothing */ 				{ [] }
    | actuals_list 				{ List.rev $1 }


actuals_list:
     expr 					{ [$1] }
   | actuals_list COMMA expr 			{ $3 :: $1 }
