{ open Parser }

let digits = ['0'-'9']+ 				(* One or more digits *)
let optDigits = ['0'-'9']* 				(* Zero or more digits *)
let sign = '+'|'-' 					(* Plus or minus sign *)
let exponent = ('e'|'E') sign? digits 			(* Exponent *)
let optExponent = exponent? 				(* Optional exponent *)



rule token = parse 
  [' ' '\t' '\r' '\n']		{ token lexbuf } 	(* White spaces *)
| "/*" 				{ comment lexbuf }	(* Comments *)
| "->" 				{ ARROW }		(* Dereference a parameter in an "Object" *)
| '+' 				{ PLUS }		(* Addition *)
| '-' 				{ MINUS }		(* Subtraction *)
| '*' 				{ TIMES }		(* Multiplication *)
| '/' 				{ DIVIDE }		(* Division *)
| ',' 				{ COMMA }		(* Sequence *)
| '(' 				{ LPAREN } 		(* Left Paren *)
| ')' 				{ RPAREN }		(* Right Paren *)
| '{' 				{ LBRACE }		(* Left Brace*) 
| '}' 				{ RBRACE }		(* Right Brace*) 
| ';' 				{ SEMI } 		(* End of Expression *)
| '=' 				{ ASSIGN }		(* Assignment *)
| "==" 				{ EQ }			(* Equal *)
| "!=" 				{ NEQ } 		(* Not equal *)
| '<' 				{ LT }			(* Less than *)
| "<=" 				{ LEQ } 		(* Less than or equal *)
| ">" 				{ GT }			(* Greater than *)
| ">=" 				{ GEQ }			(* Greater than or equal *)
| "&&"				{ AND }			(* And logical operator *)
| "||"				{ OR }			(* Or logical operator *)
| "cmp"				{ COMPARE}		(* String comparison operator *)
| "concat"			{ CONCAT}		(* Concatenation of string constants *)
| "return"			{ RETURN }		(* Return value *)
| "while"			{ WHILE }		(* Loop *)
| "if" 				{ IF }			(* Conditional *)
| "else" 			{ ELSE }		(* Alternate control flow *)
| "Init" 			{ INIT }		(* Initialization *)
| eof 				{ EOF }			(* End of File *)

| '\"' [^ '\"']* '\"'					as lxm { STRCONST(lxm) }	(* String constant*)
| ( (digits '.'? optDigits optExponent) |
    ('.'? digits optExponent) )				as lxm { LITERAL(lxm) }		(* Integer or float *)
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* 	as lxm { ID(lxm) } 		(* Name of a variable*)
| _ as char 			{ raise (Failure("illegal character " ^ Char.escaped char)) }


and comment = parse "*/" { token lexbuf }
| _ { comment lexbuf }